@@ -3,11 +3,13 @@ from . import Layers
 from . import Model
 from . import sul_tool
 from . import sulio 
+from . import sulplotter
 
 __all__ = [
 	'DataReader',
 	'Layers',
 	'Model',
-	'sul_tool'
-	'sulio'
+	'sul_tool',
+	'sulio',
+	'sulplotter'
 ]
\ No newline at end of file
